# coding=utf-8

"""Define hooks for code_aster repositories."""

import os
import os.path as osp

from aslint.logger import logger
from repository_api import get_branch, get_main_branch, get_repo_name, repository_type

from ..ext_utils import get_changed_files, is_admin
from .generic import NOOK, OK, branch_checker, branch_user_commit_hook, branch_user_push_hook

# authorized branches names (+ vNN)
AUTH_BRANCH = ("^default$", "^edf/", "^asterxx/", "^v([0-9]+(\.[0-9]+)?(_smeca)?)$")

UNAUTH_BRANCH_GIT = ("^main", "^v([0-9]+(\.[0-9]+)?)")

# pylint: disable=invalid-name
repo_branch_checker = branch_checker(authorized=AUTH_BRANCH)

pretxncommit = branch_user_commit_hook(repo_branch_checker, is_admin, check_msg=True)

pretxnchangegroup = branch_user_push_hook(repo_branch_checker, is_admin, check_msg=True)

git_branch_checker = branch_checker(authorized=".", unauthorized=UNAUTH_BRANCH_GIT)


def precommit_git(repopath, amend):
    """precommit hook for code_aster Git repository."""
    # do pretxncommit: check branch, message
    branch = get_branch(repopath)
    if git_branch_checker(branch) is NOOK:
        logger.error("You are not allowed to commit in " "branch {0!r}.".format(branch), exit=True)
    return aslint(repopath, amend=amend)


def aslint(repopath, paths=(), amend=False):
    """Call aslint before a commit: in case of known error/warning, try to
    fix it. The commit command is not cancelled.
    """
    from subprocess import call

    from aslint.base_checkers import Report, checkcontext
    from aslint.check_files import check_files, read_waf_parameters
    from aslint.config import ASCFG
    from aslint.filetype import filter_dict
    from aslint.logger import logger

    # global informations
    checkcontext.reponame = get_repo_name(repopath)
    checkcontext.branch = get_main_branch(repopath)
    revs = ("HEAD~1",) if amend else None

    # logger.title("running aslint...")
    choice = ASCFG.get("check.precommit")
    if choice == "no":
        logger.warn(
            "pre-commit checkings skipped because of 'check.precommit'"
            " value in your ~/.gitconfig file"
        )
        return OK
    elif choice == "undef":
        logger.info(
            "pre-commit: If you don't plan to submit your work "
            "you can disable these checkings by adding "
            "'check-precommit = no' in the [aster] section in "
            "your ~/.gitconfig file."
        )
        logger.info(
            "pre-commit: To hide this message you can set the value " "'check.precommit = yes'."
        )
    elif choice != "yes":
        logger.warn(
            "unknown choice {0!r} for 'check.precommit'"
            " (see ~/.gitconfig) must be 'yes' or 'no'.".format(choice)
        )
    dchg = get_changed_files(repopath, revs=revs, files=paths)
    changes = filter_dict(dchg)
    report = Report()
    builddir = ASCFG.get("waf.builddir")
    target = "release"
    if os.environ.get("BUILD") == "debug":
        builddir = "build/mpidebug" if osp.isdir("build/mpidebug") else "build/mpi"
        target = "debug"
    wafc4che = osp.join(builddir, "c4che", target + "_cache.py")

    c4che = read_waf_parameters(wafc4che)

    check_files(report, changes, c4che)
    fixed = False
    if report.hasError():
        # run the fixers and remove the messages from the report
        dcmd = report.getFixerCommands()
        for eid, cmd in list(dcmd.items()):
            fixed = True
            logger.warn("fixing error {0}...".format(eid))
            call(cmd, shell=True)
            report.remove(eid)
    # check for remaining errors
    if not report.hasError():
        nbchg = sum([len(i) for i in changes.values()])
        if nbchg > 0:
            logger.info("pre-commit: aslint hook passed successfully")
        else:
            logger.info("pre-commit: no files to be checked")
        if fixed and repository_type(repopath) == "git":
            logger.warn("pre-commit: some files were automatically fixed and added by running:")
            toadd = dchg.get("A", []) + dchg.get("M", [])
            cmd = ["git", "add"] + toadd
            logger.info(" ".join(cmd))
            call(cmd)
    else:
        logger.info(report.to_text())
        report.report_summary()
        answ = input(
            "The commit will be not accepted because of remaining "
            "errors.\nDo you want to continue (y/n) ?"
        )
        if answ.lower() not in ("y", "o"):
            logger.info("pre-commit: aslint ended with errors.")
            return NOOK
        else:
            logger.cancel_error()
    return OK
